KNATIVE_VERSION = '1.15.0'

DOMAIN_SERVER_URL = 'https://github.com/knative/serving/releases/download/knative-v1.14.0/serving-default-domain.yaml' # not in use anymore
KNATIVE_ISTIO_CONTROLLER_URL = f'https://github.com/knative/net-istio/releases/download/knative-v{KNATIVE_VERSION}/net-istio.yaml'
KNATIVE_CRD_URL = f'https://github.com/knative/serving/releases/download/knative-v{KNATIVE_VERSION}/serving-crds.yaml'
KNATIVE_CORE_URL = f'https://github.com/knative/serving/releases/download/knative-v{KNATIVE_VERSION}/serving-core.yaml'

